<?php

namespace Drupal\pagerer;

use Drupal\Core\Url;
use Drupal\Core\Pager\PagerManager;

/**
 * Provides a manager for Pagerer, as an extension of core's PagerManager.
 */
class PagererManager extends PagerManager {

  /**
   * {@inheritdoc}
   */
  public function createPager($total, $limit, $element = 0) {
    $pager = new Pagerer($total, $limit, $this->pagerParams->findPage($element));
    $pager->setElement($element);
    $pager->setAdaptiveKeys($this->pagerParams->findAdaptiveKeys($element));
    $this->setPagerer($pager, $element);
    return $pager;
  }

  /**
   * Saves a pager to the static cache.
   *
   * @param \Drupal\pagerer\Pagerer $pager
   *   The pager.
   * @param int $element
   *   The pager index.
   */
  protected function setPagerer(Pagerer $pager, $element = 0) {
    $this->pagers[$element] = $pager;
    if (method_exists($this, 'updateGlobals')) {
      $this->updateGlobals();
    }
  }

  /**
   * Gets the URL query parameter array of a pager link.
   *
   * @param Drupal\pagerer\Pagerer $pager
   *   The pager object.
   * @param array $parameters
   *   An associative array of query string parameters to append to the pager
   *   links.
   * @param int $page
   *   The target page.
   * @param string $adaptive_keys
   *   The adaptive keys string, in the format 'L.R.X', where L is the
   *   adaptive lock to left page, R is the adaptive lock to right page,
   *   and X is the adaptive center lock for calculation of neighborhood.
   *
   * @return array
   *   The updated array of query parameters.
   */
  public function getPagererUpdatedParameters(Pagerer $pager, array $parameters, $page, $adaptive_keys = NULL) {
    $max = $this->getMaxPagerElementId();

    // Build the 'page' and 'page_ak' query parameter elements.
    // This is built based on the current page of each pager element (or NULL
    // if the pager is not set), with the exception of the requested page index
    // for the current element.
    $page_el = [];
    $page_ak = [];
    for ($i = 0; $i <= $max; $i++) {
      $page_el[$i] = isset($this->pagers[$i]) ? (($i == $pager->getElement()) ? $page : $this->pagers[$i]->getCurrentPage()) : NULL;
      $page_ak[$i] = isset($this->pagers[$i]) ? (($i == $pager->getElement()) ? $adaptive_keys : $this->pagers[$i]->getAdaptiveKeys()) : NULL;
    }

    // Build the 'page' and 'page_ak' fragments, removing unneeded trailing
    // keys.
    while (end($page_el) === NULL) {
      array_pop($page_el);
    }
    $parameters['page'] = implode(',', $page_el);
    while (end($page_ak) === NULL) {
      array_pop($page_ak);
    }
    $parameters['page_ak'] = implode(',', $page_ak);

    // Merge the updated pager query parameters, with any parameters coming
    // from the current request. In case of collision, current parameters
    // take precedence over the request ones.
    if ($current_request_query = $this->pagerParams->getQueryParameters()) {
      $parameters = array_merge($current_request_query, $parameters);
    }

    // Explicitly remove the segments if not relevant.
    if (empty($page_el)) {
      unset($parameters['page']);
    }
    if (empty($page_ak)) {
      unset($parameters['page_ak']);
    }

    return $parameters;
  }

  /**
   * Gets a pager link.
   *
   * @param Drupal\pagerer\Pagerer $pager
   *   The pager object.
   * @param array $parameters
   *   An associative array of query string parameters to append to the pager
   *   links.
   * @param int $page
   *   The target page.
   * @param string $adaptive_keys
   *   The adaptive keys string, in the format 'L.R.X', where L is the
   *   adaptive lock to left page, R is the adaptive lock to right page,
   *   and X is the adaptive center lock for calculation of neighborhood.
   * @param bool $set_query
   *   Whether the link should contain the query parameters.
   *
   * @return \Drupal\Core\Url
   *   The Url object for the link.
   */
  public function getHref(Pagerer $pager, array $parameters, $page, $adaptive_keys = NULL, $set_query = TRUE) {
    $options = $set_query ? [
      'query' => $this->getPagererUpdatedParameters($pager, $parameters, $page, $adaptive_keys),
    ] : [];
    return Url::fromRoute($pager->getRouteName(), $pager->getRouteParameters(), $options);
  }

}
